#!/bin/bash

SWAP_DEV="$1" # Swap device
TIMEOUT="$2" # In seconds
SWAPOFF="/usr/sbin/swapoff"
COUNT=0
FAILED_SPEED_COUNT=0

function used_swap() {
	local swap_dev=$1

	cat /proc/swaps | awk "{if (\$1 == "Filename" || \$1 != \"$swap_dev\") next; print \$4}" 2>/dev/null
}

function print_msg() {
	echo "$0 on $SWAP_DEV: $*" > /dev/kmsg
}

if [ ! -a "$SWAP_DEV" -o -z "$TIMEOUT" ]; then
	echo "Usage: $0 SWAP_DEVICE TIMEOUT"
	exit 1
fi

[ $TIMEOUT -eq 0 ] && exit 0

# SWAP_DEV can be the symlink
if [ -h "$SWAP_DEV" ]; then
	SWAP_DEV_READLINKED=`readlink -f $SWAP_DEV 2>/dev/null`
	[ -a "$SWAP_DEV_READLINKED" ] && SWAP_DEV="$SWAP_DEV_READLINKED"
fi

TOTAL_SWAP=`used_swap $SWAP_DEV`
if [ -z "$TOTAL_SWAP" ]; then
	print_msg "Failed to calculate used swap for $SWAP_DEV"
	# So fallback to the normal swapoff
	$SWAPOFF "$SWAP_DEV"
	SWAPOFF_RC=$?
	print_msg "$SWAPOFF finished with code $SWAPOFF_RC"
	exit $SWAPOFF_RC
fi
print_msg "Total swap used on $SWAP_DEV : $TOTAL_SWAP KiB, timeout $TIMEOUT s"

# 1. Calculate needed speed
SPEED=$((TOTAL_SWAP/TIMEOUT))
print_msg "Needed swapoff speed: $SPEED KiB/s"

FAILED_SPEED_COUNT_LIMIT=$((TIMEOUT/3))
print_msg "Average speed slowdown limit: $FAILED_SPEED_COUNT_LIMIT s"

# 2. Fork swapoff
$SWAPOFF $SWAP_DEV &
SWAPOFF_PID=$!
if [ -z "$SWAPOFF_PID" ]; then
	print_msg "Failed to get swapoff pid"
	exit 1
fi

HUNDREDTH_COUNT=0

while [ $COUNT -lt $TIMEOUT ]; do
	if [ ! -d /proc/$SWAPOFF_PID ]; then
		wait $SWAPOFF_PID > /dev/null 2>&1
		SWAPOFF_RC=$?
		print_msg "$SWAPOFF finished with code $SWAPOFF_RC"
		exit $SWAPOFF_RC
	fi
	HUNDREDTH_COUNT=$((HUNDREDTH_COUNT+1))
	if [ $HUNDREDTH_COUNT = 100 ]; then
		HUNDREDTH_COUNT=0
		COUNT=$((COUNT+1))
		USED_SWAP=`used_swap $SWAP_DEV`
		SWAP_RELEASE_SPEED=$(((TOTAL_SWAP-USED_SWAP)/COUNT))
		print_msg "Swap used:  $USED_SWAP KiB"
		print_msg "Current swapoff speed $SWAP_RELEASE_SPEED KiB/s"
		if [ $SWAP_RELEASE_SPEED -lt $SPEED ]; then
			if [ $FAILED_SPEED_COUNT -eq $FAILED_SPEED_COUNT_LIMIT ]; then
				break
			fi
			print_msg "Running slow: needed speed is $SPEED KiB/s. Awaiting speedup for $((FAILED_SPEED_COUNT_LIMIT-FAILED_SPEED_COUNT)) s"
			FAILED_SPEED_COUNT=$((FAILED_SPEED_COUNT+1))
		fi
	fi
	sleep 0.01
done

if [ -d /proc/$SWAPOFF_PID ]; then
	if [ $COUNT -eq $TIMEOUT ]; then
		print_msg "Timeout reached, kill started"
	else
		print_msg "Too slow, kill started"
	fi
	kill -9 $SWAPOFF_PID > /dev/null 2>&1
	exit 1
fi

exit 0
